﻿/*
Zip code tabulation area (ZCTA) code by Adam Shirey (adam.shirey@lexisnexis.com).
Data from US Census Bureau (census.gov):
	Census 2000 U.S. Gazetteer Files page (http://www.census.gov/geo/www/gazetteer/places2k.html)
	Direct ZCTA download link: http://www.census.gov/tiger/tms/gazetteer/zcta5.txt

To spray file (zcta5.txt assumed to exist on dropzone):
	1. Download zcta5.txt from census.gov (above) and upload to a dropzone
	2. From ECLWATCH page, click 'Spray Fixed' link
	3. Select the source dropzone and local path as appropriate.
	4. Set record length to 158
	5. Specify a destination label of 'ecldata::in::zcta5_hpcc' and click Submit
*/

layout_zcta5 := record
	string2 State;
	string64 Name;
	string9 Total_Population;
	string9 Total_Housing_Units;
	string14 Land_Area_sq_m;
	string14 Water_Area_sq_m;
	string12 Land_Area_sq_mi;
	string12 Water_Area_sq_mi; 
	string10 Latitude;
	string11 Longitude;
	string1 lf; // line feed
end;
zcta5 := dataset( '~ecldata::in::zcta5_hpcc', layout_zcta5, flat );

layout_standard := record
	zcta5.state;
	string5 zip5 := zcta5.name[1..5];
	unsigned Total_Population    := (unsigned)zcta5.Total_Population;
	unsigned Total_Housing_Units := (unsigned)zcta5.Total_Housing_Units;
	unsigned Land_Area_sq_m      := (unsigned)zcta5.Land_Area_sq_m;
	unsigned Water_Area_sq_m     := (unsigned)zcta5.Water_Area_sq_m;
	unsigned Land_Area_sq_mi     := (unsigned)zcta5.Land_Area_sq_mi;
	unsigned Water_Area_sq_mi    := (unsigned)zcta5.Water_Area_sq_mi; 
	real4 Latitude               := (real4)zcta5.Latitude;
	real4 Longitude              := (real4)zcta5.Longitude;
end;
	
// when processing on multiple nodes, distributing approximately evenly will help reduce skew
standardized := distribute(table( zcta5, layout_standard ), (integer)zip5);

layout_distance := record
	// note that an individual zip code can exist in multiple states; for example, 38852 exists in both AL and MS.
	// therefore, both the state and zip are necessary for unique identification
	string2 state1;
	string5 zip1;
	string2 state2;
	string5 zip2;
	real4 distance;
end;


layout_distance calculate_distance( layout_standard le, layout_standard ri ) := TRANSFORM
	self.state1 := le.state;
	self.zip1   := le.zip5;
	self.state2 := ri.state;
	self.zip2   := ri.zip5;
	
	// great circle distance in miles
	self.distance := 3963.0 * acos(
		sin(-le.latitude/57.2958)*sin(-ri.latitude/57.2958)
		+ cos(-le.latitude/57.2958)*cos(-ri.latitude/57.2958)*cos(-ri.longitude/57.2958 + le.longitude/57.2958)
	);
END;

// There are 33,233 zip codes; a straight cartesian join produces 33,233**2 = 1,104,432,289 combinations.
// While excessive (this could be pared down by checking for left.state=right.state or for the two states being adjacent), this is handled by 200 nodes in about a minute and a half.
with_distances := join( standardized, standardized, true, calculate_distance(left,right), all );


sorted_byDist := group(sort( with_distances, state1, zip1, distance ), state1, zip1 );
limited := sorted_byDist( distance <= 100 ); // all combinations where the zip/state pairs are less than or equal to 100 miles apart
output( count(limited), named('num_within_100mi') ); // how many city pairs exist within 100 miles of each other? (note that this is bi-directional)

// What is the distance from Chicago, IL to New York, NY?
chi_ny := sorted_byDist( state1='IL', zip1='60611', state2='NY', zip2='10001' );
output(chi_ny[1].distance, named('distance_chicago_newyork'));


// How many cities (zip codes) are within 15 miles of Duluth, MN? Zip codes taken from USPS.com, four of which are officially associated with PO Boxes
duluth_zips := ['55701','55801','55802','55803','55804','55805','55806','55807','55808','55810','55811','55812','55814','55815','55816'];
duluth_matches := with_distances( zip1 in duluth_zips, zip2 not in duluth_zips, distance <= 15 );
duluth_deduped := dedup( duluth_matches, state2, zip2, all );
duluth_slim := table( duluth_deduped, {state2, zip2, distance} );
output( duluth_slim, named('duluth_neighbors'), all );